@@ -1,47 +1,22 @@
-
 /*
- * $Id$
- *
- *
- * SQUID Web Proxy Cache          http://www.squid-cache.org/
- * ----------------------------------------------------------
- *
- *  Squid is the result of efforts by numerous individuals from
- *  the Internet community; see the CONTRIBUTORS file for full
- *  details.   Many organizations have provided support for Squid's
- *  development; see the SPONSORS file for full details.  Squid is
- *  Copyrighted (C) 2001 by the Regents of the University of
- *  California; see the COPYRIGHT file for full details.  Squid
- *  incorporates software developed and/or copyrighted by other
- *  sources; see the CREDITS file for full details.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111, USA.
+ * Copyright (C) 1996-2020 The Squid Software Foundation and contributors
  *
+ * Squid software is distributed under GPLv2+ license and includes
+ * contributions from numerous individuals and organizations.
+ * Please see the COPYING and CONTRIBUTORS files for details.
  */
 
 #ifndef SQUID_ICAPSERVICEREP_H
 #define SQUID_ICAPSERVICEREP_H
 
-#include "cbdata.h"
-#include "FadingCounter.h"
-#include "adaptation/Service.h"
 #include "adaptation/forward.h"
-#include "adaptation/Initiator.h"
 #include "adaptation/icap/Elements.h"
+#include "adaptation/Initiator.h"
+#include "adaptation/Service.h"
 #include "base/AsyncJobCalls.h"
+#include "cbdata.h"
 #include "comm.h"
+#include "FadingCounter.h"
 #include "pconn.h"
 #include <deque>
 
@@ -81,10 +56,10 @@ class OptXact;
  * auto-destroyed by refcounting when no longer used.
  */
 
-
 class ServiceRep : public RefCountable, public Adaptation::Service,
-        public Adaptation::Initiator
+    public Adaptation::Initiator
 {
+    CBDATA_CLASS(ServiceRep);
 
 public:
     typedef RefCount<ServiceRep> Pointer;
@@ -100,18 +75,18 @@ public:
     bool availableForNew() const; ///< a new transaction may start communicating with the service
     bool availableForOld() const; ///< a transaction notified about connection slot availability may start communicating with the service
 
-    virtual Initiate *makeXactLauncher(HttpMsg *virginHeader, HttpRequest *virginCause);
+    virtual Initiate *makeXactLauncher(Http::Message *virginHeader, HttpRequest *virginCause, AccessLogEntry::Pointer &alp);
 
     void callWhenAvailable(AsyncCall::Pointer &cb, bool priority = false);
     void callWhenReady(AsyncCall::Pointer &cb);
 
     // the methods below can only be called on an up() service
-    bool wantsUrl(const String &urlPath) const;
-    bool wantsPreview(const String &urlPath, size_t &wantedSize) const;
+    bool wantsUrl(const SBuf &urlPath) const;
+    bool wantsPreview(const SBuf &urlPath, size_t &wantedSize) const;
     bool allows204() const;
     bool allows206() const;
     Comm::ConnectionPointer getConnection(bool isRetriable, bool &isReused);
-    void putConnection(const Comm::ConnectionPointer &conn, bool isReusable, const char *comment);
+    void putConnection(const Comm::ConnectionPointer &conn, bool isReusable, bool sendReset, const char *comment);
     void noteConnectionUse(const Comm::ConnectionPointer &conn);
     void noteConnectionFailed(const char *comment);
 
@@ -135,6 +110,9 @@ public: // treat these as private, they are for callbacks only
     // receive either an ICAP OPTIONS response header or an abort message
     virtual void noteAdaptationAnswer(const Answer &answer);
 
+    Security::ContextPointer sslContext;
+    Security::SessionStatePointer sslSession;
+
 private:
     // stores Prepare() callback info
 
@@ -143,7 +121,7 @@ private:
         AsyncCall::Pointer callback;
     };
 
-    typedef Vector<Client> Clients;
+    typedef std::vector<Client> Clients;
     // TODO: rename to theUpWaiters
     Clients theClients; // all clients waiting for a call back
 
@@ -208,7 +186,6 @@ private:
 
     mutable bool wasAnnouncedUp; // prevent sequential same-state announcements
     bool isDetached;
-    CBDATA_CLASS2(ServiceRep);
 };
 
 class ModXact;
@@ -219,7 +196,7 @@ class ConnWaiterDialer: public NullaryMemFunT<ModXact>
 public:
     typedef NullaryMemFunT<ModXact> Parent;
     ServiceRep::Pointer theService;
-    ConnWaiterDialer(const CbcPointer<ModXact> &xact, Parent::Method aHandler);
+    ConnWaiterDialer(const CbcPointer<Adaptation::Icap::ModXact> &xact, Adaptation::Icap::ConnWaiterDialer::Parent::Method aHandler);
     ConnWaiterDialer(const Adaptation::Icap::ConnWaiterDialer &aConnWaiter);
     ~ConnWaiterDialer();
 };
@@ -228,3 +205,4 @@ public:
 } // namespace Adaptation
 
 #endif /* SQUID_ICAPSERVICEREP_H */
+
